// RiftToSharedMemory.cpp : Defines the entry point for the console application.
/*
Loosely based on OVR's SensorBoxTest code.

License: Apache 2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#include "OVR.h"
#include "Kernel/OVR_String.h"
#include <stdint.h>

using namespace OVR;

struct Generic6DOF
{
	int32_t data_id;
	float yaw;
	float pitch;
	float roll;
	float x;
	float y;
	float z;
};

double RadToDeg(double rad)
{
	return rad * 180.0 / 3.14159265359;
}

int main(int argc, char* argv[])
{
    OVR::System::Init();

    Ptr<DeviceManager> pManager = 0;
    Ptr<HMDDevice>     pHMD = 0;
    Ptr<SensorDevice>  pSensor = 0;
    SensorFusion       FusionResult;

     // *** Oculus HMD & Sensor Initialization

    // Create DeviceManager and first available HMDDevice from it.
    // Sensor object is created from the HMD, to ensure that it is on the
    // correct device.

    pManager = *DeviceManager::Create();

    int         detectionResult = IDCONTINUE;
    const char* detectionMessage;

    do 
    {
        // Release Sensor/HMD in case this is a retry.
        pSensor.Clear();
        pHMD.Clear();

        pHMD  = *(pManager->EnumerateDevices<HMDDevice>().CreateDevice());
        if (pHMD)
        {
            pSensor = *pHMD->GetSensor();
        }

        // If there was a problem detecting the Rift, display appropriate message.
        detectionResult  = IDCONTINUE;        

        if (!pHMD && !pSensor)
            detectionMessage = "Oculus Rift not detected.";
        else if (!pHMD)
            detectionMessage = "Oculus Sensor detected; HMD Display not detected.";
        else if (!pSensor)
            detectionMessage = "Oculus HMD Display detected; Sensor not detected.";
        else
            detectionMessage = 0;

		if (detectionMessage)
		{
			detectionResult = IDCANCEL;
			printf(detectionMessage);
			printf("\nDetection will be attempted again in 1 second.\n");
			Sleep(1000);
		}

    } while (detectionResult != IDCONTINUE);
    
    if (pSensor)
	{
        FusionResult.AttachToSensor(pSensor);
		printf("Connected to sensor.\n");
		FusionResult.SetYawCorrectionEnabled(true);
	}

	Generic6DOF *data_6dof = NULL;
	HANDLE h_generic6dof_file = CreateFileMapping(
		INVALID_HANDLE_VALUE,	// use paging file
		NULL,					// default security
		PAGE_READWRITE,			// read only access
		0,						// maximum object size (high-order DWORD)
		sizeof(Generic6DOF)*1,		// maximum object size (low-order DWORD)
		TEXT("FPGeneric"));				// name of mapping object

	if (h_generic6dof_file != NULL)										// Could not create file mapping object
	{
		data_6dof = (Generic6DOF*) MapViewOfFile(h_generic6dof_file,			// handle to map object
			FILE_MAP_WRITE,									// read/write permission
			0,
			0,
			sizeof(Generic6DOF)*1);

		if (data_6dof == NULL)										// Could not map view of file
		{
			printf("Error: Could not map file view.\n");
			CloseHandle(h_generic6dof_file);
		}
	}

	while(true)
	{
		// *** Per Frame
		// Get orientation quaternion to control view
		Quatf q = FusionResult.GetOrientation();

		// Get Euler angles from quaternion, in specified axis rotation order.
		float yaw, pitch, roll;
        // Note: Using the OVR SDK built in GetEulerAngles, as pitch approaches 90 degrees, roll will quickly increase
        // even if not turning your head. This can be addressed by using a smaller SingularityRadius, such as 0.005f.
		q.GetEulerAngles<Axis_Y, Axis_X, Axis_Z>(&yaw, &pitch, &roll);
		yaw = -yaw;
		roll = -roll;
		//QuatToEuler(q, &roll, &pitch, &yaw);
		data_6dof[0].yaw = yaw;
		data_6dof[0].pitch = pitch;
		data_6dof[0].roll = roll;

		printf("\rYaw  %.01fdeg\t  Pitch  %.01fdeg\t  Roll  %.01fdeg            ", RadToDeg(yaw), RadToDeg(pitch), RadToDeg(roll));

		Sleep(5);
	}
	
	if (data_6dof != NULL)
	{
		UnmapViewOfFile(data_6dof);
		data_6dof = NULL;
	}
	if (h_generic6dof_file != NULL)
	{
		CloseHandle(h_generic6dof_file);
		h_generic6dof_file = NULL;
	}

    // *** Shutdown
    pSensor.Clear();
    pHMD.Clear();
	pManager.Clear();
	FusionResult.AttachToSensor(NULL);

    OVR::System::Destroy();

	return 0;
}
